from almaqso import download_archive, analysis


if __name__ == '__main__':
    download_archive(4, 'catalog/test_2.json')
    analysis('.', casapath='/usr/local/casa/casa-6.6.1-17-pipeline-2024.1.0.8/bin/casa')  # Specify the path to the CASA binary



import os
import sys
import shutil
from pathlib import Path
sys.path.append('../almaqso')  # isort:skip
from almaqso import download_archive, analysis


CURRENT_DIR = Path(__file__).parent
CASA_PATH = '/usr/local/casa/casa-6.6.1-17-pipeline-2024.1.0.8/bin/casa'  # Path to CASA. Please modify this if necessary.
DATA_PATH = './uid___A002_Xd68367_X9885'
TARFILE = './2018.1.01575.S_uid___A002_Xd68367_X9885.asdm.sdm.tar'

# Edit the following constants.
DOWNLOAD = True  # True: Download the tar file, False: Use the existing tar file


def test_download():
    if DOWNLOAD:
        shutil.rmtree(TARFILE, ignore_errors=True)
        download_archive(4, 'catalog/test_2.json')
        assert os.path.exists(TARFILE)


def test_analysis():
    shutil.rmtree(DATA_PATH, ignore_errors=True)

    analysis('.', casapath=CASA_PATH)



import os
import glob
import subprocess


def _run_casa_cmd(casa: str, mpicasa: str, n_core: int,
                  cmd: str, verbose: bool) -> None:
    """
    Run a CASA command.

    Args:
        casa (str): Path to the CASA executable. Provide full path even if it is in the PATH for using MPI CASA.
        mpicasa (str): Path to the MPI CASA executable.
        n_core (int): Number of cores to run with MPI CASA.
        cmd (str): CASA command to run.
        verbose (bool): Print the STDOUT of the CASA commands.

    Returns:
        None
    """
    if mpicasa is not None:
        exe = mpicasa
        options = ['-n', str(n_core), casa, '--nologger', '--nogui', '-c', cmd]
    else:
        exe = casa
        options = ['--nologger', '--nogui', '-c', cmd]
    try:
        result = subprocess.run(
            [exe] + options,
            check=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True
        )
        if verbose:
            print(f"STDOUT for {cmd}:", result.stdout)
            print(f"STDERR for {cmd}:", result.stderr)
    except subprocess.CalledProcessError as e:
        if verbose:
            print(f"Error while executing {cmd}:")
            print(f"Return Code: {e.returncode}")
            print(f"STDOUT: {e.stdout}")
            print(f"STDERR: {e.stderr}")


def _calibration(casa_options: map) -> None:
    """
    Run the calibration steps.

    Args:
        casa_options (map): Dictionary containing the CASA options.

    Returns:
        None
    """
    scriptfile = glob.glob('*.scriptForCalibration.py')[0]

    with open(scriptfile, 'r') as f:
        syscalcheck = f.readlines().copy()[21]

    scriptfile_part = scriptfile.replace('.py', '.part.py')
    with open(scriptfile_part, 'w') as f:
        if syscalcheck.split(':')[1].split("'")[1] == \
                'Application of the bandpass and gain cal tables':
            f.write(
                'mysteps = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16]'+'\n'
            )
        else:
            f.write(
                'mysteps = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17]'+'\n'
            )
        f.write('applyonly = True'+'\n')
        f.write(f"execfile(\"{scriptfile}\", globals())\n")

    _run_casa_cmd(
        cmd=f"execfile('{scriptfile_part}', globals())",
        **casa_options
    )


def _check_severe_error() -> bool:
    """
    Check the severe error in the log files.

    Args:
        None

    Returns:
        bool: True if the severe error is found, False otherwise
    """
    log_files = glob.glob('*.log')
    result = False

    for log_file in log_files:
        with open(log_file, 'r') as f:
            lines = f.readlines()
        for i, line in enumerate(lines):
            if 'SEVERE' in line:
                print(f'SEVERE error is found in {log_file} (line: {i+1})')
                result = True

    return result


def analysis(tardir: str, casapath: str, mpicasa: bool = False,
             n_core: int = 2, skip: bool = True, verbose: bool = False) -> None:
    """
    Run the analysis of the QSO data.

    Args:
        tardir (str): Directory containing the `*.asdm.sdm.tar` files.
        casapath (str): Path to the CASA executable. Provide full path even if it is in the PATH for using MPI CASA.
        mpicasa (bool): Use MPI CASA. Default is False.
        n_core (int): Number of cores to use for the analysis. Default is 8.
        skip (bool): Skip the analysis if the output directory exists. Default is True.
        verbose (bool): Print the STDOUT of the CASA commands when no errors occur. Default is False.

    Returns:
        None
    """
    asdm_files = [file for file in os.listdir(f'{tardir}') if file.endswith('.asdm.sdm.tar')]
    almaqso_dir = os.path.split(os.path.dirname(os.path.abspath(__file__)))[0]

    if mpicasa:
        mpicasa = os.path.dirname(casapath) + '/mpicasa'
    else:
        mpicasa = None

    casa_options = {
        'casa': casapath,
        'mpicasa': mpicasa,
        'n_core': n_core,
        'verbose': verbose
    }

    severe_error_list = []

    for asdm_file in asdm_files:
        asdmname = 'uid___' + \
                (asdm_file.split('_uid___')[1]).replace('.asdm.sdm.tar', '')
        print(f'Processing {asdmname}')

        if os.path.exists(asdmname) and skip:
            print(f'{asdmname}: analysis already done and skip')
            continue
        if os.path.exists(asdmname):
            print(f'{asdmname}: analysis already done but reanalyzed')
        else:
            os.makedirs(asdmname)

        os.chdir(asdmname)
        os.system(f'tar -xf ../{asdm_file}')

        # Create calibration script
        cmd = f"sys.path.append('{almaqso_dir}');" + \
            "from almaqso._qsoanalysis import _make_script;" + \
            f"_make_script('{asdm_file}')"
        _run_casa_cmd(cmd=cmd, **casa_options)

        # Calibration
        _calibration(casa_options)

        # Remove target
        cmd = f"sys.path.append('{almaqso_dir}');" + \
            "from almaqso._qsoanalysis import _remove_target;" + \
            "_remove_target()"
        _run_casa_cmd(cmd=cmd, **casa_options)

        # Check severe error
        if _check_severe_error():
            severe_error_list.append(asdmname)

        os.chdir('..')
        print(f'Processing {asdmname} is done.')

    print('#' * 80)
    print('All processing is done.')

    if len(severe_error_list) > 0:
        print('The following data have SEVERE errors:')
        for severe_error in severe_error_list:
            print(severe_error)
    else:
        print('No severe errors are found.')



import os
import shutil
import glob
from casatasks import *
import analysisUtils as aU
import almaqa2csg as csg
from casampi.MPICommandClient import MPICommandClient


def _make_script(tarfilename: str) -> None:
    """
    Make a CASA script for the QSO analysis.

    Args:
        tarfilename (str): File name of ASDM tar data.

    Returns:
        None
    """
    try:
        print(f'analysisUtils of {aU.version()} will be used.')
    except Exception:
        raise Exception('analysisUtils is not found')

    # Step 1. Import the ASDM file
    projID = tarfilename.split('_uid___')[0]
    asdmfile = glob.glob(
            f'{os.getcwd()}/{os.path.basename(projID)}/*/*/*/raw/*'
            )[0]
    visname = (os.path.basename(asdmfile)).replace('.asdm.sdm', '.ms')

    kw_importasdm = {
        'asdm': asdmfile,
        'vis': visname,
        'asis': 'Antenna Station Receiver Source CalAtmosphere CalWVR CorrelatorMode SBSummary',
        'bdfflags': True,
        'lazy': True,
        'flagbackup': False,
    }

    shutil.rmtree(kw_importasdm['vis'], ignore_errors=True)
    importasdm(**kw_importasdm)

    casalog.post('almaqso: Import the ASDM file is done.')

    # Step 2. Generate a calibration script
    if not os.path.exists(
            f'./{visname}.scriptForCalibration.py'
            ):
        casalog.post('almaqso: Generate a calibration script.')
        refant = aU.commonAntennas(visname)
        csg.generateReducScript(
            msNames=visname,
            refant=refant[0],
            corrAntPos=False,
            useCalibratorService=False,
            useLocalAlmaHelper=False,
        )

    casalog.post('almaqso: Generated calibration script is done.')


def _remove_target() -> None:
    """
    Remove the target fields from the measurement set.

    Args:
        None

    Returns:
        None
    """
    client = MPICommandClient()
    client.set_log_mode('redirect')
    client.start_services()

    visname = glob.glob('*.ms.split')[0]
    fields = aU.getFields(visname)
    fields_target = aU.getTargetsForIntent(visname)
    fields_cal = list(set(fields) - set(fields_target))

    shutil.rmtree(visname + '.org', ignore_errors=True)
    shutil.move(visname, visname + '.org')

    if visname + '.listobs' in os.listdir():
        shutil.move(visname + '.listobs', visname + '.org.listobs')

    kw_split = {
        'vis': visname + '.org',
        'outputvis': visname,
        'field': ', '.join(fields_cal),
        'datacolumn': 'all',
    }

    shutil.rmtree(kw_split['outputvis'], ignore_errors=True)
    shutil.rmtree(kw_split['outputvis'] + '.flagversions', ignore_errors=True)

    command = f'mstransform("{kw_split["vis"]}", "{kw_split["outputvis"]}",' + \
            f'field="{kw_split["field"]}", datacolumn="{kw_split["datacolumn"]}")'

    client.push_command_request(command,block,target_server,parameters)
